@@ -1,51 +1,21 @@
 package Bitcoin::Crypto::Role::Key;
-$Bitcoin::Crypto::Role::Key::VERSION = '2.007';
+$Bitcoin::Crypto::Role::Key::VERSION = '3.000';
 use v5.10;
 use strict;
 use warnings;
-use Crypt::PK::ECC;
-use Scalar::Util qw(blessed);
 use Mooish::AttributeBuilder -standard;
 use Types::Common -sigs, -types;
 
 use Bitcoin::Crypto::Types -types;
 use Bitcoin::Crypto::Constants;
 use Bitcoin::Crypto::Util qw(get_key_type);
-use Bitcoin::Crypto::Helpers qw(ensure_length);    # loads Math::BigInt
+use Bitcoin::Crypto::Helpers qw(ensure_length ecc);
 use Bitcoin::Crypto::Exception;
 
-sub __create_key
-{
-	my ($entropy) = @_;
-
-	return $entropy
-		if blessed($entropy) && $entropy->isa('Crypt::PK::ECC');
-
-	my $is_private = get_key_type $entropy;
-
-	Bitcoin::Crypto::Exception::KeyCreate->raise(
-		'invalid entropy data passed to key creation method'
-	) unless defined $is_private;
-
-	$entropy = ensure_length $entropy, Bitcoin::Crypto::Constants::key_max_length
-		if $is_private;
-
-	my $key = Crypt::PK::ECC->new();
-
-	Bitcoin::Crypto::Exception::KeyCreate->trap_into(
-		sub {
-			$key->import_key_raw($entropy, Bitcoin::Crypto::Constants::curve_name);
-		}
-	);
-
-	return $key;
-}
-
 use Moo::Role;
 
 has param 'key_instance' => (
-	isa => InstanceOf ['Crypt::PK::ECC'],
-	coerce => \&__create_key,
+	isa => ByteStr,
 );
 
 has param 'purpose' => (
@@ -64,10 +34,23 @@ requires qw(
 sub BUILD
 {
 	my ($self) = @_;
+	my $entropy = $self->key_instance;
+
+	my $is_private = get_key_type $entropy;
 
 	Bitcoin::Crypto::Exception::KeyCreate->raise(
 		'trying to create key from unknown key data'
-	) unless $self->key_instance->is_private == $self->_is_private;
+	) unless $is_private == $self->_is_private;
+
+	Bitcoin::Crypto::Exception::KeyCreate->raise(
+		'invalid entropy data passed to key creation method'
+	) unless defined $is_private;
+
+	if ($is_private) {
+		Bitcoin::Crypto::Exception::KeyCreate->raise(
+			'private key is not valid'
+		) unless ecc->verify_private_key(ensure_length $entropy, Bitcoin::Crypto::Constants::key_max_length);
+	}
 }
 
 signature_for has_purpose => (
@@ -82,13 +65,6 @@ sub has_purpose
 	return !$self->purpose || $self->purpose == $purpose;
 }
 
-# __create_key for object usage
-sub _create_key
-{
-	shift;
-	goto \&__create_key;
-}
-
 signature_for raw_key => (
 	method => Object,
 	positional => [Maybe [Enum [qw(private public public_compressed)]], {default => undef}],
@@ -97,24 +73,30 @@ signature_for raw_key => (
 sub raw_key
 {
 	my ($self, $type) = @_;
+	my $is_private = $self->_is_private;
 
-	unless (defined $type) {
-		$type = 'public_compressed';
-		if ($self->_is_private) {
-			$type = 'private';
-		}
-		elsif ($self->does('Bitcoin::Crypto::Role::Compressed') && !$self->compressed) {
-			$type = 'public';
-		}
+	$type = $is_private ? 'private' : 'public'
+		unless defined $type;
+
+	if ($type eq 'private') {
+		Bitcoin::Crypto::Exception::KeyCreate->raise(
+			'cannot create private key from a public key'
+		) unless $is_private;
+
+		return ensure_length $self->key_instance, Bitcoin::Crypto::Constants::key_max_length;
 	}
-	return $self->key_instance->export_key_raw($type);
-}
+	else {
+		my $compressed = $self->does('Bitcoin::Crypto::Role::Compressed') ? $self->compressed : !!1;
+		$compressed = !!1 if $type eq 'public_compressed';
 
-sub curve_order
-{
-	my ($self) = @_;
+		my $key = $self->key_instance;
+		$key = ecc->create_public_key($key)
+			if $is_private;
 
-	return Math::BigInt->from_hex($self->key_instance->curve2hash->{order});
+		return ecc->compress_public_key($key, $compressed);
+	}
+
+	# no need to check for invalid input, since we have a signature with enum
 }
 
 1;
